<?php
/**
 * @file
 * Publications webform component
 */

/**
 * Implements _webform_defaults_component().
 */
function _webform_defaults_publications() {
  return array(
    'name' => '',
    'form_key' => NULL,
    'pid' => 0,
    'weight' => 0,
    'value' => '',
    'mandatory' => 0,
    'extra' => array(
      'disabled' => FALSE,
      'private' => FALSE,
      'key_component' => 0,
      'search' => FALSE,
      'limit' => 4,
      'show_state' => TRUE,
      'radius' => 50,
      'unit' => 'km',
      'country' => 'CA',
      'radius' => 50,
      'empty' => 'No publications exist in close range to the postal code you have provided.',
      'analysis_limit' => 0,
    ),
  );
}

/**
 * Implements _webform_theme_component().
 */
function _webform_theme_publications() {
  return array(
    'webform_display_publications' => array(
      'render element' => 'element',
    ),
  );
}

/**
 * Custom Theme function for collected publications data.
 */
function theme_webform_display_publications($variables) {
  $element = $variables['element'];
  $value = $element['#value'];
  return $value;
}

/**
 * Implements _webform_edit_component().
 */
function _webform_edit_publications($component) {
  // Get other components for this form from DB.
  $sql = 'SELECT * FROM {webform_component} where nid = :nid ';
  $args = array(':nid' => $component['nid']);
  // Add cid to query if it exists
  if (array_key_exists('cid', $component)) {
    $sql .= 'AND cid != :cid ';
    $args[':cid'] = $component['cid'];
  }
  $sql .= "ORDER BY name ASC";
  $result = db_query($sql, $args);

  // Dump components into options array.
  $options = array();
  $options[0] = 'None';
  foreach ($result as $row) {
    $options[$row->cid] = $row->name;
  }

  $form = array();
  $form['publications'] =  array(
    '#type' => 'fieldset',
    '#title' => t('Publications'),
    '#weight' => 1,
    '#collapsible' => TRUE,
  );

  $form['publications']['key_component'] = array(
    '#type' => 'select',
    '#title' => t('Key Component'),
    '#description' => t('Select another component in this webform that will contain a zip/postal code.  If filled in on a page prior to this publications component, this zip/postal code will reduce the list of available publications presented to the user. If none is selected, all publications will be presented.'),
    '#weight' => 2,
    '#options' => $options,
    '#default_value' => $component['extra']['key_component'],
    '#parents' => array('extra', 'key_component'),
  );

  $form['publications']['search'] = array(
    '#type' => 'checkbox',
    '#title' => t('Provide an autocomplete search field'),
    '#return_value' => 1,
    '#default_value' => $component['extra']['search'],
    '#weight' => 2,
    '#parents' => array('extra', 'search'),
  );

  $form['publications']['country'] = array(
    '#type' => 'select',
    '#title' => t('Country'),
    '#description' => t('Country to populate publications list from.'),
    '#default_value' => $component['extra']['country'],
    '#options' => webform_publications_countries(),
    '#multiple' => FALSE,
    '#weight' => 3,
    '#parents' => array('extra', 'country'),
  );

  $form['publications']['unit'] = array(
    '#type' => 'select',
    '#title' => t('Unit'),
    '#description' => t('Unit of measurement used to determine distances between zip/postal codes.'),
    '#default_value' => $component['extra']['unit'],
    '#options' => array(
      'km' => t('Kilometers'),
      'miles' => t('Miles'),
    ),
    '#multiple' => FALSE,
    '#weight' => 4,
    '#parents' => array('extra', 'unit'),
  );

  $form['publications']['radius'] = array(
    '#type' => 'select',
    '#title' => t('Radius'),
    '#description' => t('Range in chose units from given zip/postal code that publications should be within.'),
    '#default_value' => $component['extra']['radius'],
    '#options' => array(
      25 => t('25'),
      50 => t('50'),
      75 => t('75'),
      100 => t('100'),
    ),
    '#multiple' => FALSE,
    '#weight' => 5,
    '#parents' => array('extra', 'radius'),
  );

  $form['publications']['limit'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum # of publications to show'),
    '#default_value' => $component['extra']['limit'],
    '#weight' => 6,
    '#parents' => array('extra', 'limit'),
    '#element_validate' => array('element_validate_integer_positive'),
  );

  $form['publications']['show_state'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display the state or province of a publication'),
    '#return_value' => 1,
    '#default_value' => $component['extra']['show_state'],
    '#weight' => 7,
    '#parents' => array('extra', 'show_state'),
  );

  $form['publications']['empty'] =  array(
    '#type' => 'textarea',
    '#title' => t('No results text'),
    '#description' => t('The message added here will be displayed to users who enter a zip or postal that is not in range of any publications within the database.  HTML is permitted.'),
    '#weight' => 8,
    '#default_value' => $component['extra']['empty'],
    '#parents' => array('extra', 'empty'),
  );

  $options = array('No limit');
  for ($i = 1; $i <= 30; $i++) {
    $options[$i] = $i;
  }
  $form['publications']['analysis_limit'] = array(
    '#title'         => t('Show analysis for publications contacted more than this many times'),
    '#type'          => 'select',
    '#options'       => $options,
    '#multiple'      => FALSE,
    '#default_value' => $component['extra']['analysis_limit'],
    '#description'   => t('Limit publication analysis to only show publications who were contacted more than this many times.'),
    '#parents' => array('extra', 'analysis_limit'),
    '#weight' => 13,
  );

  $form['display']['disabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disabled'),
    '#return_value' => 1,
    '#description' => t('Make this field non-editable. Useful for setting an unchangeable default value.'),
    '#weight' => 9,
    '#default_value' => $component['extra']['disabled'],
    '#parents' => array('extra', 'disabled'),
  );

  return $form;
}

/**
 * Implements _webform_render_component().
 */
function _webform_render_publications($component, $value = NULL, $filter = TRUE) {
  $name = $component['name'];
  $required = $component['mandatory'];
  $weight = $component['weight'];
  $display = $component['extra']['title_display'];
  $disabled = $component['extra']['disabled'];
  $search = $component['extra']['search'];
  $show_state = $component['extra']['show_state'];
  $country = $component['extra']['country'];

  $element = array(
    '#theme_wrappers' => array('webform_element'),
    '#pre_render' => array(),
  );

  $checkboxes = array(
    '#type' => 'checkboxes',
    '#options' => array(),
    '#title' => $filter ? _webform_filter_xss($name) : $name,
    '#title_display' => $display ? $display : 'before',
    '#required' => $required,
    '#weight' => $weight,
    '#translatable' => array('title', 'description'),
    '#attributes' => array('class' => array('webform-prefill-exclude')),
  );

  // Set default value.
  if (!empty($value)) {
    // Set the value as an array.
    $checkboxes['#default_value'] = array();
    foreach ((array) $value as $key => $option_value) {
      $checkboxes['#default_value'][] = $option_value;
    }
  } else {
    $checkboxes['#attributes']['checked'] = 'checked';
  }

  // Define the webform element
  if ($search) {
    $element['search'] = array(
      '#type' => 'textfield',
      '#title' => t('Search'),
      '#autocomplete_path' => 'webform_publications/search/' . $country . '/' . $show_state,
      '#webform_component' => $component,
      '#ajax' => array(
        'callback' => 'webform_publications_ajax_callback',
        'wrapper' => 'webform_publications_' . $component['cid'] . '_replace',
      ),
      '#attributes' => array('class' => array('webform-prefill-exclude')),
    );
    $element['checkboxes'] = $checkboxes;
  }
  else {
    $element = $checkboxes;
  }

  // Handle disabling.
  if ($disabled) {
    if ($filter) {
      $element['#attributes']['readonly'] = 'readonly';
    }
    else {
      $element['#disabled'] = TRUE;
    }
  }

  return $element;
}

/**
 * Implements _webform_submit_component().
 */
function _webform_submit_publications($component, $value) {
  if (isset($value['checkboxes'])) {
    return $value['checkboxes'];
  }
  else {
    return $value;
  }
}

/**
 * Implements _webform_display_component().
 */
function _webform_display_publications($component, $value, $format = 'html') {
  $wrapper = $format == 'html' ? 'webform_element' : 'webform_element_text';
  return array(
    '#title' => $component['name'],
    '#weight' => $component['weight'],
    '#theme' => 'webform_display_publications',
    '#theme_wrappers' => array($wrapper),
    '#format' => $format,
    '#value' => _webform_publications_values_to_string($value),
    '#translatable' => array('title'),
  );
}

/**
 * Implements _webform_analysis_component().
 */
function _webform_analysis_publications($component, $sids = array()) {
  $query = db_select('webform_submitted_data', 'wsd', array('fetch' => PDO::FETCH_ASSOC))
    ->fields('wsd', array('data'))
    ->condition('nid', $component['nid'])
    ->condition('cid', $component['cid']);

  if (count($sids)) {
    $query->condition('sid', $sids, 'IN');
  }

  $summary = array();
  $result = $query->execute();
  // Collate results by target
  foreach ($result as $data) {
    $target = webform_publications_load($data['data'])->title;
    if ($target == '') {
      $target = 'No publication';
    }
    if (isset($summary[$target])) {
      $summary[$target]++;
    }
    else {
      $summary[$target] = 1;
    }
  }

  // If there's a limit set, dump the other targets into a single bucket
  $limit = isset($component['extra']['analysis_limit'])
    ? $component['extra']['analysis_limit']
    : 0;
  if ($limit) {
    $other = 0;
    foreach ($summary as $target => $count) {
      if ($count <= $limit) {
        $other += $count;
        unset($summary[$target]);
      };
    }
    $summary['Publications contacted ' . $limit . ' times or fewer'] = $other;
  }

  // Sort by number of times contacted, descending
  arsort($summary);

  // Convert array to table format
  $rows = array();
  foreach ($summary as $target => $count) {
    $rows[] = array(
      $target,
      $count,
    );
  }

  // Spit out the table
  $header = array(t('Target'), t('Count'));
  return array(
    'table_header' => $header,
    'table_rows' => $rows,
  );
}

/**
 * Implements _webform_table_component().
 */
function _webform_table_publications($component, $value) {
  return _webform_publications_values_to_string($value);
}

/**
 * Implements _webform_csv_headers_component().
 */
function _webform_csv_headers_publications($component, $export_options) {
  $header = array();
  $header[0] = '';
  $header[1] = '';
  $header[2] = $component['name'];
  return $header;
}

/**
 * Implements _webform_csv_data_component().
 */
function _webform_csv_data_publications($component, $export_options, $value) {
  return _webform_publications_values_to_string($value);
}

/**
 * Creates string from array of publication values.
 */
function _webform_publications_values_to_string($value) {
  // Initialize string.
  $publications = '';

  if (is_array($value)) {
    $processed = array();

    // Build array that just contains selected email addresses.
    foreach ($value as $key => $contained) {
      if (!is_int($contained) && $contained !== '0') {
        $processed[$key] = $contained;
      }
    }

    // Get count of selected email addresses.
    $count = count($processed);
    $i = 0;

    // Format each value of proceesed array and add it
    // to publications string.
    foreach ($processed as $key => $contained) {

      $i++;

      // Add publication email to string.
      $publications = $publications . $key;

      if ($i < $count) {
        // Add comma to string for all items except last.
        $publications = $publications . ', ';
      }
    }
  }

  return check_plain(empty($publications) ? '' : $publications);
}
